import {
  ACCESS_GROUP,
  PERMISSION,
  VDOM_SCOPE,
  ADMIN_TYPE,
} from "fos-common/fortigate/state";
/**
 * Permission pair type.
 */
export type PermissionPairType = {
  [K in ACCESS_GROUP]?: PERMISSION | boolean;
};

/**
 * Scope type.
 */
export type ScopeType = {
  [K in ADMIN_TYPE]: VDOM_SCOPE;
};

/**
 * Permission type base.
 */
interface PermissionTypeBase {
  /**
   * Scope type.
   */
  scope?: ScopeType;
}

/**
 * Permission with access group.
 */
interface PermissionWithAccessGroup extends PermissionTypeBase {
  /**
   * Permission for access groups.
   */
  accessGroup: PermissionPairType[];
  /**
   * Permission for one of access groups.
   */
  accessGroupOneOf?: PermissionPairType[];
}

/**
 * Permission with one of access groups.
 */
interface PermissionWithAccessGroupOneOf extends PermissionTypeBase {
  /**
   * Permission for one of access groups.
   */
  accessGroupOneOf: PermissionPairType[];
  /**
   * Permission for access groups.
   */
  accessGroup?: PermissionPairType[];
}

/**
 * Permission type.
 */
export type PermissionType =
  | PermissionWithAccessGroup
  | PermissionWithAccessGroupOneOf;

/**
 * Route permission config.
 */
export type AngularJSRoutePermissionObject = {
  /**
   * URL
   */
  url: string;
  /**
   * Allow non eager param.
   */
  allowNonEagerParam?: boolean;
  /**
   * AngularJS config.
   */
  config: {
    /**
     * File path.
     */
    path?: string;
    /**
     * Legacy base dialog.
     */
    legacy_base_dialog?: boolean;
    /**
     * Lagecy base page.
     */
    legacy_base?: boolean;
    /**
     * Template
     */
    template?: string;
    /**
     * Controller name.
     */
    controller?: string;
    /**
     * CSS file.
     */
    css?: string[];
    /**
     * Reload on serach.
     */
    reloadOnSearch?: boolean;
    /**
     * Loader resolve file name.
     */
    loader_resolve?: string[];
    /**
     * Permission.
     */
    permission: PermissionType;
  };
};

export const NG_ROUTE_CONFIG: {
  routes: AngularJSRoutePermissionObject[];
};
